﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Xunit;
using HttpSysManager.HttpAPI;
using ProcessHacker.Native.Security.AccessControl;
using ProcessHacker.Native.Security;
using System.Runtime.InteropServices;
using System.Net;
using System.Security.Cryptography.X509Certificates;

namespace HttpSysManager.Tests
{
	public class TestAcl : IDisposable
	{
		readonly string _Url;
		public string Url
		{
			get
			{
				return _Url;
			}
		}
		public TestAcl()
		{
			_Url = "http://+:9393/";
			HttpAPIManager manager = new HttpAPIManager();
			manager.SetUrlAcl(_Url, SecurityDescriptor.FromSDDL("D:(A;;GX;;;WD)"));
		}
		#region IDisposable Members

		public void Dispose()
		{
			HttpAPIManager manager = new HttpAPIManager();
			manager.RemoveUrlAcl(_Url);
		}

		#endregion
	}

	public class TestClass
	{
		[Fact]
		public void CanQueryAcls()
		{
			HttpAPIManager manager = new HttpAPIManager();
			var testUrl = "http://+:9399/CanQueryAclsTest";
			manager.SetUrlAcl(testUrl, SecurityDescriptor.FromSDDL("D:(A;;;;;WD)"));
			try
			{
				var containsTemporaryUrl = manager.GetAclInfo().Any(url => url.Prefix.Contains("CanQueryAclsTest"));
				Assert.True(containsTemporaryUrl, "You should be able to retrieve Acls");
			}
			finally
			{
				manager.RemoveUrlAcl(testUrl);
			}
		}
		[Fact]
		public void CanAddAndRemoveAcl()
		{
			var testUrl = "http://+:9393/";
			HttpAPIManager manager = new HttpAPIManager();
			manager.SetUrlAcl(testUrl, SecurityDescriptor.FromSDDL("D:(A;;GX;;;WD)"));
			try
			{
				var acl = manager.GetAclInfo(testUrl);
				Assert.NotNull(acl);
				var fakeAcl = manager.GetAclInfo("unknown");
				Assert.Null(fakeAcl);
			}
			finally
			{
				manager.RemoveUrlAcl(testUrl);
			}
			var oldAcl = manager.GetAclInfo(testUrl);
			Assert.Null(oldAcl);
		}

		[Fact]
		public void CanSetUrlTwiceAndSetToNullToRemove()
		{
			var testUrl = "http://+:9394/";
			HttpAPIManager manager = new HttpAPIManager();
			manager.SetUrlAcl(testUrl, SecurityDescriptor.FromSDDL("D:(A;;GX;;;WD)"));
			manager.SetUrlAcl(testUrl, SecurityDescriptor.FromSDDL("D:(A;;;;;WD)"));
			Assert.Equal("D:(A;;;;;WD)", manager.GetAclInfo(testUrl).SecurityDescriptor.ToString());
			manager.SetUrlAcl(testUrl, null);
			Assert.Null(manager.GetAclInfo(testUrl));
		}
		[Fact]
		public void CanBrowseCertificates()
		{
			HttpAPIManager manager = new HttpAPIManager();
			Assert.True(1 <= manager.GetSSLInfos().Count());
		}
		[Fact]
		public void CanSetCertificates()
		{
			var testEndpoint = new IPEndPoint(IPAddress.Parse("127.0.0.1"), 462);
			HttpAPIManager manager = new HttpAPIManager();
			manager.SetSSLInfo(testEndpoint, new SSLInfo(TestCertificate.Thumbprint));
			try
			{
				var sslInfo = manager.GetSSLInfo(testEndpoint);
				Assert.NotNull(sslInfo);
			}
			finally
			{
				manager.RemoveSSLInfo(testEndpoint);
				var sslInfo = manager.GetSSLInfo(testEndpoint);
				Assert.Null(sslInfo);
			}
		}

		[Fact]
		public void CanSetSSLInfoTwiceAndSetToNullToRemove()
		{
			var testEndpoint = new IPEndPoint(IPAddress.Parse("127.0.0.1"), 464);
			var info = new SSLInfo(TestCertificate.Thumbprint);

			HttpAPIManager manager = new HttpAPIManager();

			manager.SetSSLInfo(testEndpoint, info);
			try
			{
				Assert.NotNull(manager.GetSSLInfo(testEndpoint));
				Assert.Equal(false, manager.GetSSLInfo(testEndpoint).NegotiateClientCert);
				info.NegotiateClientCert = true;
				manager.SetSSLInfo(testEndpoint, info);
				Assert.NotNull(manager.GetSSLInfo(testEndpoint));
				Assert.Equal(true, manager.GetSSLInfo(testEndpoint).NegotiateClientCert);
			}
			finally
			{
				manager.SetSSLInfo(testEndpoint, null);
				Assert.Null(manager.GetSSLInfo(testEndpoint));
			}
		}
		public static X509Certificate2 TestCertificate
		{
			get
			{
				return new X509Certificate2("../../Certificates/HttpSysManager.pfx", "", X509KeyStorageFlags.PersistKeySet | X509KeyStorageFlags.MachineKeySet);
			}
		}
		//[Fact]
		//public void CanAddEmptyAcl()
		//{
		//    var testUrl = "http://test2:3827/";
		//    HttpAPIManager manager = new HttpAPIManager();
		//    try
		//    {
		//        manager.SetUrlAcl(testUrl, SecurityDescriptor.Empty);
		//        manager.GetAclInfo(testUrl);
		//    }
		//    finally
		//    {
		//        manager.RemoveUrlAcl(testUrl);
		//    }
		//}
	}
}
