﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using System.Security;
using Microsoft.Win32.SafeHandles;
using System.Threading;
using System.Runtime.ConstrainedExecution;
using System.Security.Permissions;
using System.ComponentModel;
using ProcessHacker.Native.Api;
using System.Security.Cryptography.X509Certificates;
using System.Net;

namespace HttpSysManager.HttpAPI
{
	public class Constants
	{
		public const int HTTP_INITIALIZE_CONFIG = 0x00000002;
		public const int HTTP_INITIALIZE_SERVER = 0x00000001;
		public static readonly HTTPAPI_VERSION HTTPAPI_VERSION_1 = new HTTPAPI_VERSION()
		{
			HttpApiMajorVersion = 1,
			HttpApiMinorVersion = 0
		};
		public static readonly HTTPAPI_VERSION HTTPAPI_VERSION_2 = new HTTPAPI_VERSION()
		{
			HttpApiMajorVersion = 2,
			HttpApiMinorVersion = 0
		};
		public const int SDDL_REVISION_1 = 1;
		public const uint HTTP_SERVICE_CONFIG_SSL_FLAG_USE_DS_MAPPER = 0x00000001;
		public const uint HTTP_SERVICE_CONFIG_SSL_FLAG_NEGOTIATE_CLIENT_CERT = 0x00000002;
		public const uint HTTP_SERVICE_CONFIG_SSL_FLAG_NO_RAW_FILTER = 0x00000004;
	}

	public class Security
	{
		[DllImport("aclui.dll")]
		public static extern bool EditSecurity(
			[In] IntPtr hWnd,
			[MarshalAs(UnmanagedType.Interface)]
            [In] ISecurityInformation SecurityInformation
			);

		[DllImport("advapi32.dll")]
		public static extern bool ConvertStringSecurityDescriptorToSecurityDescriptorW([In] [MarshalAs(UnmanagedType.LPWStr)]  string securityDescriptor,
  [In]   int StringSDRevision,
  [Out]  out IntPtr SecurityDescriptor,
  [Out]  out ulong SecurityDescriptorSize);

		[DllImport("advapi32.dll")]
		public static extern bool ConvertSecurityDescriptorToStringSecurityDescriptor([In] IntPtr securityDescriptor,
  [In]   int StringSDRevision,
			[In] SecurityInformation securityInformation,
  [Out]  out IntPtr str,
  [Out]  out int strLen);

		[DllImport("Cryptui.dll")]
		public static extern IntPtr CryptUIDlgSelectCertificateW(IntPtr selectCertificateStruct);
		[DllImport("Crypt32.dll")]
		public static extern bool CertFreeCertificateContext(IntPtr pCert);

		public static X509Certificate2 SelectCertificate()
		{
			bool storeOpened = false;
			var store = new X509Store(StoreName.My, StoreLocation.LocalMachine);
			IntPtr result = IntPtr.Zero;
			IntPtr intputPtr = IntPtr.Zero;
			var array = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(IntPtr)));
			try
			{
				store.Open(OpenFlags.ReadOnly);
				storeOpened = true;
				Marshal.WriteIntPtr(array, store.StoreHandle);


				int size = Marshal.SizeOf(typeof(CRYPTUI_SELECTCERTIFICATE_STRUCTW));
				CRYPTUI_SELECTCERTIFICATE_STRUCTW input = new CRYPTUI_SELECTCERTIFICATE_STRUCTW();

				input.dwSize = (uint)size;
				input.cPropSheetPages = 0;
				input.dwDontUseColumn = 0;
				input.dwFlags = 0;
				input.hSelectedCertStore = IntPtr.Zero;
				input.hwndParent = IntPtr.Zero;
				input.pDisplayCallback = IntPtr.Zero;
				input.pFilterCallback = IntPtr.Zero;
				input.pvCallbackData = IntPtr.Zero;
				input.rghDisplayStores = array;
				input.cDisplayStores = 1;
				input.rghStores = IntPtr.Zero;
				input.cStores = 0;
				input.rgPropSheetPages = IntPtr.Zero;
				input.szDisplayString = null;
				input.szTitle = null;


				intputPtr = Marshal.AllocHGlobal(size);
				Marshal.StructureToPtr(input, intputPtr, false);
				result = Security.CryptUIDlgSelectCertificateW(intputPtr);
				if(result == IntPtr.Zero)
					return null;
				return new X509Certificate2(result);
			}
			finally
			{
				if(storeOpened)
					store.Close();
				if(intputPtr != IntPtr.Zero)
				{
					Marshal.DestroyStructure(intputPtr, typeof(CRYPTUI_SELECTCERTIFICATE_STRUCTW));
					Marshal.FreeHGlobal(intputPtr);
				}
				if(array != IntPtr.Zero)
					Marshal.FreeHGlobal(array);
				if(result != IntPtr.Zero)
					Security.CertFreeCertificateContext(result); //X509Certificate2 duplicate the handle internally
			}
		}
	}

	[StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode)]
	public class CRYPTUI_SELECTCERTIFICATE_STRUCTW
	{
		public uint dwSize;
		public IntPtr hwndParent;
		public uint dwFlags;
		public string szTitle;
		public uint dwDontUseColumn;
		public string szDisplayString;
		public IntPtr pFilterCallback;
		public IntPtr pDisplayCallback;
		public IntPtr pvCallbackData;
		public uint cDisplayStores;
		public IntPtr rghDisplayStores;
		public uint cStores;
		public IntPtr rghStores;
		public uint cPropSheetPages;
		public IntPtr rgPropSheetPages;
		public IntPtr hSelectedCertStore;
	}
	//[Flags]
	//public enum SecurityInformation : uint
	//{
	//    Owner = 0x00000001,
	//    Group = 0x00000002,
	//    Dacl = 0x00000004,
	//    Sacl = 0x00000008,
	//    Label = 0x00000010,

	//    ProtectedDacl = 0x80000000,
	//    ProtectedSacl = 0x40000000,
	//    UnprotectedDacl = 0x20000000,
	//    UnprotectedSacl = 0x10000000
	//}

	public enum SeObjectType : int
	{
		Unknown = 0,
		FileObject,
		Service,
		Printer,
		RegistryKey,
		LmShare,
		KernelObject,
		WindowObject,
		DsObject,
		DsObjectAll,
		ProviderDefinedObject,
		WmiGuidObject,
		RegistryWow6432Key
	}

	[Flags]
	public enum SiAccessFlags : int
	{
		Specific = 0x00010000,
		General = 0x00020000,
		Container = 0x00040000,
		Property = 0x00080000
	}

	public enum SiCallbackMessage : uint
	{
		Release = 1,
		Create = 2,
		InitDialog = 0x8001
	}

	[Flags]
	public enum SiObjectInfoFlags : int
	{
		EditPerms = 0x00000000,
		EditOwner = 0x00000001,
		EditAudits = 0x00000002,
		Container = 0x00000004,
		ReadOnly = 0x00000008,
		Advanced = 0x00000010,
		Reset = 0x00000020,
		OwnerReadOnly = 0x00000040,
		EditProperties = 0x00000080,
		Recurse = 0x00000100,
		NoAclProtect = 0x00000200,
		NoTreeApply = 0x00000400,
		PageTitle = 0x00000800,
		ServerIsDc = 0x00001000,
		ResetDaclTree = 0x00004000,
		ResetSaclTree = 0x00008000,
		ObjectGuid = 0x00010000,
		EditEffective = 0x00020000,
		ResetDacl = 0x00040000,
		ResetSacl = 0x00080000,
		ResetOwner = 0x00100000,
		NoAdditionalPermission = 0x00200000,
		ViewOnly = 0x00400000,
		PermsElevationRequired = 0x01000000,
		AuditsElevationRequired = 0x02000000,
		OwnerElevationRequested = 0x04000000,
		MayWrite = 0x10000000,
		All = EditPerms | EditOwner | EditAudits
	}

	public enum SiPageType : int
	{
		Perm,
		AdvPerm,
		Audit,
		Owner,
		Effective,
		TakeOwnership
	}

	[StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode)]
	public struct SiAccess
	{
		public IntPtr Guid;
		public int Mask;
		public IntPtr Name; // string
		public SiAccessFlags Flags;
	}

	[StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode)]
	public struct SiInheritType
	{
		public IntPtr Guid;
		public int Flags;
		public IntPtr Name; // string
	}

	[StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode)]
	public struct SiObjectInfo
	{
		public SiObjectInfoFlags Flags;
		public IntPtr Instance;
		public IntPtr ServerName; // string
		public IntPtr ObjectName; // string
		public IntPtr PageTitle; // string
		public Guid ObjectType;
	}

	public class UnManagedOutputStruct<T> : IDisposable
	{
		public UnManagedOutputStruct(IntPtr output)
		{
			this.output = output;
			_Value = (T)Marshal.PtrToStructure(output, typeof(T));
		}

		public UnManagedOutputStruct()
		{
		}

		private readonly T _Value;
		public T Value
		{
			get
			{
				return _Value;
			}
		}

		IntPtr output;
		#region IDisposable Members

		public void Dispose()
		{
			if(output != IntPtr.Zero)
			{
				Marshal.FreeHGlobal(output);
			}
		}

		#endregion
	}
	public class Functions
	{
		[DllImport("httpapi.dll", CallingConvention = CallingConvention.StdCall, ExactSpelling = true, SetLastError = true)]
		public static extern uint HttpCreateHttpHandle(out SafeCloseHandle pReqQueueHandle, uint options);
		[DllImport("httpapi.dll", SetLastError = true)]
		public static extern uint HttpInitialize(
			 HTTPAPI_VERSION Version,
			 uint Flags,
			 IntPtr pReserved);
		[DllImport("httpapi.dll", SetLastError = true)]
		public static extern uint HttpTerminate(
			 uint Flags,
			 IntPtr pReserved);
		[DllImport("httpapi.dll", SetLastError = true)]
		public static extern uint HttpSetServiceConfiguration(
			 IntPtr ServiceIntPtr,
			 HTTP_SERVICE_CONFIG_ID ConfigId,
			 IntPtr pConfigInformation,
			 int ConfigInformationLength,
			 IntPtr pOverlapped);
		[DllImport("httpapi.dll", SetLastError = true, ExactSpelling = true, EntryPoint = "HttpSetServiceConfiguration")]
		public static extern uint HttpSetACLServiceConfiguration(
			 [In] IntPtr ServiceIntPtr,
			 [In] HTTP_SERVICE_CONFIG_ID ConfigId,
			 [In] ref HTTP_SERVICE_CONFIG_URLACL_SET pConfigInformation,
			 [In] int ConfigInformationLength,
			 [In] IntPtr pOverlapped);
		[DllImport("httpapi.dll", SetLastError = true, ExactSpelling = true, EntryPoint = "HttpSetServiceConfiguration")]
		public static extern uint HttpSetSSLServiceConfiguration(
			 [In] IntPtr ServiceIntPtr,
			 [In] HTTP_SERVICE_CONFIG_ID ConfigId,
			 [In] ref HTTP_SERVICE_CONFIG_SSL_SET pConfigInformation,
			 [In] int ConfigInformationLength,
			 [In] IntPtr pOverlapped);


		[DllImport("httpapi.dll", SetLastError = true)]
		public static extern uint HttpDeleteServiceConfiguration(
			 IntPtr ServiceIntPtr,
			 HTTP_SERVICE_CONFIG_ID ConfigId,
			 IntPtr pConfigInformation,
			 int ConfigInformationLength,
			 IntPtr pOverlapped);
		[DllImport("httpapi.dll", SetLastError = true, ExactSpelling = true, EntryPoint = "HttpDeleteServiceConfiguration")]
		public static extern uint HttpDeleteACLServiceConfiguration(
			 [In] IntPtr ServiceIntPtr,
			 [In] HTTP_SERVICE_CONFIG_ID ConfigId,
			 [In] ref HTTP_SERVICE_CONFIG_URLACL_SET pConfigInformation,
			 [In] int ConfigInformationLength,
			 [In] IntPtr pOverlapped);
		[DllImport("httpapi.dll", SetLastError = true, ExactSpelling = true, EntryPoint = "HttpDeleteServiceConfiguration")]
		public static extern uint HttpDeleteSSLServiceConfiguration(
			 [In] IntPtr ServiceIntPtr,
			 [In] HTTP_SERVICE_CONFIG_ID ConfigId,
			 [In] ref HTTP_SERVICE_CONFIG_SSL_SET pConfigInformation,
			 [In] int ConfigInformationLength,
			 [In] IntPtr pOverlapped);

		[DllImport("httpapi.dll", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Unicode, ExactSpelling = true, SetLastError = true)]
		public static extern uint HttpCreateRequestQueue(HTTPAPI_VERSION version, string pName, SECURITY_ATTRIBUTES pSecurityAttributes, uint flags, out HttpRequestQueueV2Handle pReqQueueHandle);
		[DllImport("httpapi.dll", CallingConvention = CallingConvention.StdCall, ExactSpelling = true, SetLastError = true)]
		public static extern uint HttpCloseRequestQueue(IntPtr pReqQueueHandle);
		//http://www.pinvoke.net/default.aspx/httpapi.httpqueryserviceconfiguration
		[DllImport("httpapi.dll", SetLastError = true)]
		public static extern uint HttpQueryServiceConfiguration(
			 IntPtr ServiceIntPtr,
			 HTTP_SERVICE_CONFIG_ID ConfigId,
			 IntPtr pInputConfigInfo,
			 int InputConfigInfoLength,
			 IntPtr pOutputConfigInfo,
			 int OutputConfigInfoLength,
			 [Optional()]
     out int pReturnLength,
			 IntPtr pOverlapped);
		public static bool HttpQuerySimpleServiceConfiguration<TInput, TOuput>(IntPtr servicePtr, HTTP_SERVICE_CONFIG_ID configId, TInput inputConfig, out UnManagedOutputStruct<TOuput> outputConfig) where TOuput : new()
		{
			IntPtr input;
			int size;
			int outputSize;
			IntPtr output = IntPtr.Zero;
			outputConfig = new UnManagedOutputStruct<TOuput>();
			CreateBuffer(inputConfig, out input, out size);
			try
			{
				var result = HttpQueryServiceConfiguration(servicePtr
						, configId
						, input
						, size
						, IntPtr.Zero
						, 0
						, out outputSize
						, IntPtr.Zero);
				if(result == (int)ErrorCodes.ERROR_NO_MORE_ITEMS)
					return false;
				if(result == (int)ErrorCodes.ERROR_FILE_NOT_FOUND)
					return false;
				if(result != (int)ErrorCodes.ERROR_INSUFFICIENT_BUFFER)
					Win32.Throw((int)result);
				output = Marshal.AllocHGlobal(outputSize);
				result = HttpQueryServiceConfiguration(servicePtr
						, configId
						, input
						, size
						, output
						, outputSize
						, out outputSize
						, IntPtr.Zero);
				if(result != 0)
					Win32.Throw((int)result);
				outputConfig = new UnManagedOutputStruct<TOuput>(output);
				return true;
			}
			finally
			{
				if(input != IntPtr.Zero)
				{
					Marshal.DestroyStructure(input, typeof(HTTP_SERVICE_CONFIG_URLACL_QUERY));
					Marshal.FreeHGlobal(input);
				}
			}
		}
		public static bool HttpQueryACLServiceConfiguration(IntPtr servicePtr, HTTP_SERVICE_CONFIG_URLACL_QUERY inputConfig, out UnManagedOutputStruct<HTTP_SERVICE_CONFIG_URLACL_SET> outputConfig)
		{
			return HttpQuerySimpleServiceConfiguration(servicePtr, HTTP_SERVICE_CONFIG_ID.HttpServiceConfigUrlAclInfo, inputConfig, out outputConfig);
		}

		public static bool HttpQuerySSLServiceConfiguration(IntPtr servicePtr, HTTP_SERVICE_CONFIG_SSL_QUERY inputConfig, out UnManagedOutputStruct<HTTP_SERVICE_CONFIG_SSL_SET> outputConfig)
		{
			return HttpQuerySimpleServiceConfiguration(servicePtr, HTTP_SERVICE_CONFIG_ID.HttpServiceConfigSSLCertInfo, inputConfig, out outputConfig);
		}

		static protected void CreateBuffer(object initObject, out IntPtr ptr, out int lenght)
		{
			lenght = Marshal.SizeOf(initObject);
			ptr = Marshal.AllocHGlobal(lenght);
			Marshal.StructureToPtr(initObject, ptr, false);
		}


		public static IDisposable InitializeConfig()
		{
			return new HttpInitialize(Constants.HTTPAPI_VERSION_2, Constants.HTTP_INITIALIZE_CONFIG);
		}


	}

	class HttpInitialize : IDisposable
	{
		uint flags;
		public HttpInitialize(HTTPAPI_VERSION version, uint flags)
		{
			this.flags = flags;
			uint retVal = (uint)ErrorCodes.ERROR_SUCCESS;

			retVal = Functions.HttpInitialize(version, flags, IntPtr.Zero);
			if((uint)ErrorCodes.ERROR_SUCCESS != retVal)
			{
				throw new Win32Exception((int)retVal);
			}
		}

		#region IDisposable Members

		public void Dispose()
		{
			Functions.HttpTerminate(flags, IntPtr.Zero);
		}

		#endregion
	}

	public enum ErrorCodes
	{
		ERROR_INSUFFICIENT_BUFFER = 122,
		ERROR_INVALID_HANDLE = 6,
		ERROR_SUCCESS = 0,
		ERROR_NO_MORE_ITEMS = 259,
		ERROR_FILE_NOT_FOUND = 2,
		ERROR_ALREADY_EXISTS = 183,
		ERROR_NO_SUCH_LOGON_SESSION = 1312
	}

	[StructLayout(LayoutKind.Sequential)]
	public struct HTTP_SERVICE_CONFIG_URLACL_SET
	{
		public HTTP_SERVICE_CONFIG_URLACL_KEY KeyDesc;
		public HTTP_SERVICE_CONFIG_URLACL_PARAM ParamDesc;

		public HTTP_SERVICE_CONFIG_URLACL_SET(string prefix, string securityDescription)
		{
			KeyDesc = new HTTP_SERVICE_CONFIG_URLACL_KEY(prefix);
			ParamDesc = new HTTP_SERVICE_CONFIG_URLACL_PARAM(securityDescription);
		}
		public override string ToString()
		{
			return KeyDesc.pUrlPrefix + " " + ParamDesc.pStringSecurityDescriptor;
		}
	}

	[StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode)]
	public struct HTTP_SERVICE_CONFIG_URLACL_PARAM
	{

		[MarshalAs(UnmanagedType.LPWStr)]
		public string pStringSecurityDescriptor;

		public HTTP_SERVICE_CONFIG_URLACL_PARAM(string securityDescriptor)
		{
			pStringSecurityDescriptor = securityDescriptor;
		}
	}


	public enum HTTP_SERVICE_CONFIG_ID
	{
		HttpServiceConfigIPListenList = 0,
		HttpServiceConfigSSLCertInfo,
		HttpServiceConfigUrlAclInfo,
		HttpServiceConfigTimeout,
		HttpServiceConfigCache,
		HttpServiceConfigSslSniCertInfo,
		HttpServiceConfigMax

	}
	[StructLayout(LayoutKind.Sequential)]
	public struct HTTP_SERVICE_CONFIG_SSL_QUERY
	{
		public HTTP_SERVICE_CONFIG_QUERY_TYPE QueryDesc;
		public HTTP_SERVICE_CONFIG_SSL_KEY KeyDesc;
		public uint dwToken;
	}
	[StructLayout(LayoutKind.Sequential)]
	public struct HTTP_SERVICE_CONFIG_URLACL_QUERY
	{
		public HTTP_SERVICE_CONFIG_QUERY_TYPE QueryDesc;
		public HTTP_SERVICE_CONFIG_URLACL_KEY KeyDesc;
		public uint dwToken;
	}
	[StructLayout(LayoutKind.Sequential)]
	struct HTTP_SERVICE_CONFIG_IP_LISTEN_QUERY
	{
		public int AddrCount;
		public IntPtr AddrList;
	}
	[StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode)]
	public struct HTTP_SERVICE_CONFIG_URLACL_KEY
	{
		[MarshalAs(UnmanagedType.LPWStr)]
		public string pUrlPrefix;

		public HTTP_SERVICE_CONFIG_URLACL_KEY(string urlPrefix)
		{
			pUrlPrefix = urlPrefix;
		}
	}
	[StructLayout(LayoutKind.Sequential)]
	public struct HTTP_SERVICE_CONFIG_SSL_KEY
	{
		public IntPtr pIpPort;

		public HTTP_SERVICE_CONFIG_SSL_KEY(IPEndPoint endpoint)
		{
			this.pIpPort = CreateEndpointPtr(endpoint);
		}
		private static IntPtr CreateEndpointPtr(IPEndPoint endpoint)
		{
			if(endpoint == null)
				return IntPtr.Zero;
			var socketAddr = endpoint.Serialize();
			var socketBytes = ToBytes(socketAddr);
			IntPtr arrayPtr = Marshal.AllocHGlobal(socketBytes.Length);
			Marshal.Copy(socketBytes, 0, arrayPtr, socketBytes.Length);
			return arrayPtr;
		}

		private static byte[] ToBytes(SocketAddress socketAddr)
		{
			byte[] array = new byte[socketAddr.Size];
			for(int i = 0 ; i < socketAddr.Size ; i++)
				array[i] = socketAddr[i];
			return array;
		}
	}
	public enum HTTP_SERVICE_CONFIG_QUERY_TYPE
	{
		HttpServiceConfigQueryExact = 0,
		HttpServiceConfigQueryNext,
		HttpServiceConfigQueryMax
	}
	[StructLayout(LayoutKind.Sequential)]
	public struct HTTP_SERVICE_CONFIG_SSL_SET
	{
		public HTTP_SERVICE_CONFIG_SSL_KEY KeyDesc;
		public HTTP_SERVICE_CONFIG_SSL_PARAM ParamDesc;


		public HTTP_SERVICE_CONFIG_SSL_SET(IPEndPoint endpoint)
		{
			ParamDesc = default(HTTP_SERVICE_CONFIG_SSL_PARAM);
			KeyDesc = new HTTP_SERVICE_CONFIG_SSL_KEY(endpoint);
		}
	}

	//http://wikiencyclopedia.net/4.0/DEVDIV_TFS/Dev10/Releases/RTMRel/ndp/cdf/src/WCF/Tools/WSATConfig/Configuration/HttpWrapper.cs/1305376/HttpWrapper.cs
	[StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode)]
	public struct HTTP_SERVICE_CONFIG_SSL_PARAM
	{
		public int SslHashLength;
		public IntPtr pSslHash;
		public Guid AppId;
		[MarshalAs(UnmanagedType.LPWStr)]
		public string pSslCertStoreName;
		public uint DefaultCertCheckMode;
		public int DefaultRevocationFreshnessTime;
		public int DefaultRevocationUrlRetrievalTimeout;
		[MarshalAs(UnmanagedType.LPWStr)]
		public string pDefaultSslCtlIdentifier;
		[MarshalAs(UnmanagedType.LPWStr)]
		public string pDefaultSslCtlStoreName;
		public uint DefaultFlags;
	}

	[SuppressUnmanagedCodeSecurity]
	public sealed class HttpRequestQueueV2Handle : CriticalHandleZeroOrMinusOneIsInvalid
	{
		private int disposed;
		private HttpRequestQueueV2Handle()
		{
		}
		public IntPtr DangerousGetHandle()
		{
			return this.handle;
		}
		protected override bool ReleaseHandle()
		{
			return this.IsInvalid || Interlocked.Increment(ref this.disposed) != 1 || Functions.HttpCloseRequestQueue(this.handle) == 0u;
		}
	}

	[StructLayout(LayoutKind.Sequential)]
	public class SECURITY_ATTRIBUTES
	{
		public int nLength = 12;
		public SafeLocalMemHandle lpSecurityDescriptor = new SafeLocalMemHandle(IntPtr.Zero, false);
		public bool bInheritHandle;
	}
	[SuppressUnmanagedCodeSecurity]
	[HostProtection(SecurityAction.LinkDemand, MayLeakOnAbort = true)]
	public sealed class SafeLocalMemHandle : SafeHandleZeroOrMinusOneIsInvalid
	{
		public SafeLocalMemHandle()
			: base(true)
		{
		}
		[SecurityPermission(SecurityAction.LinkDemand, UnmanagedCode = true)]
		public SafeLocalMemHandle(IntPtr existingHandle, bool ownsHandle)
			: base(ownsHandle)
		{
			base.SetHandle(existingHandle);
		}
		[DllImport("advapi32.dll", BestFitMapping = false, CharSet = CharSet.Auto, SetLastError = true)]
		public static extern bool ConvertStringSecurityDescriptorToSecurityDescriptor(string StringSecurityDescriptor, int StringSDRevision, out SafeLocalMemHandle pSecurityDescriptor, IntPtr SecurityDescriptorSize);
		[ReliabilityContract(Consistency.WillNotCorruptState, Cer.Success)]
		[DllImport("kernel32.dll")]
		private static extern IntPtr LocalFree(IntPtr hMem);
		protected override bool ReleaseHandle()
		{
			return SafeLocalMemHandle.LocalFree(this.handle) == IntPtr.Zero;
		}
	}


	public struct HTTPAPI_VERSION
	{
		public ushort HttpApiMajorVersion;
		public ushort HttpApiMinorVersion;
	}

	public class SafeNetHandles
	{
		[DllImport("kernel32.dll", ExactSpelling = true, SetLastError = true)]
		public static extern bool CloseHandle(IntPtr handle);
	}
	[SuppressUnmanagedCodeSecurity]
	public sealed class SafeCloseHandle : CriticalHandleZeroOrMinusOneIsInvalid
	{
		private const string SECURITY = "security.dll";
		private const string ADVAPI32 = "advapi32.dll";
		private const string HTTPAPI = "httpapi.dll";
		private int _disposed;
		private SafeCloseHandle()
		{
		}
		public IntPtr DangerousGetHandle()
		{
			return this.handle;
		}
		protected override bool ReleaseHandle()
		{
			return this.IsInvalid || Interlocked.Increment(ref this._disposed) != 1 || SafeNetHandles.CloseHandle(this.handle);
		}
		[ReliabilityContract(Consistency.WillNotCorruptState, Cer.Success)]
		public void Abort()
		{
			this.ReleaseHandle();
			base.SetHandleAsInvalid();
		}
	}
}
